//
//	FontBaseVerticalLayout.java
//
//	version  1.0  98/01/15
//	platform Java 1.0
//	author   Mitsumasa IWATSUKI
//

package layout;

import java.awt.*;
import java.util.Hashtable;
import java.util.Vector;

public class FontBaseVerticalLayout implements LayoutManager {
	public static final int LEFT = 0;
	public static final int CENTER = 1;
	public static final int RIGHT = 2;
	public static final int TOP = 3;
	public static final int MIDDLE = 4;
	public static final int BOTTOM = 5;

	protected Hashtable compMap;
	protected int halign;
	protected int valign;
	protected int gap;
	protected String baseString;
	protected static final String defaultBaseString = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	public FontBaseVerticalLayout() {
		construct(CENTER, MIDDLE, 4, defaultBaseString);
	}

	public FontBaseVerticalLayout(int align) {
		construct(align, 4, defaultBaseString);
	}

	public FontBaseVerticalLayout(int halign, int valign) {
		construct(halign, valign, 4, defaultBaseString);
	}

	public FontBaseVerticalLayout(int halign, int valign, int gap) {
		construct(halign, valign, gap, defaultBaseString);
	}

	public FontBaseVerticalLayout(String baseString) {
		construct(CENTER, MIDDLE, 4, baseString);
	}

	public FontBaseVerticalLayout(int align, String baseString) {
		construct(align, 4, baseString);
	}

	public FontBaseVerticalLayout(int halign, int valign, String baseString) {
		construct(halign, valign, 4, baseString);
	}

	public FontBaseVerticalLayout(int halign, int valign, int gap, String baseString) {
		construct(halign, valign, gap, baseString);
	}

	public void setAlignment(int align) {
		switch (align) {
		case LEFT: case CENTER: case RIGHT:
			this.halign = align;
			break;
		case TOP: case MIDDLE: case BOTTOM:
			this.valign = align;
			break;
		}
	}

	public void setAlignment(int halign, int valign) {
		if (LEFT <= halign && halign <= RIGHT)
			this.halign = halign;

		if (TOP <= valign && valign <= BOTTOM)
			this.valign = valign;
	}

	public int getAlignment() {
		return halign;
	}

	public int getHorizontalAlignment() {
		return halign;
	}

	public int getVerticalAlignment() {
		return valign;
	}

	public void setGap(int gap) {
		this.gap = gap;
	}

	public int getGap() {
		return gap;
	}

	public void setBaseString(String baseString) {
		setBaseStringImpl(baseString);
	}

	public String getBaseString() {
		return baseString;
	}

	public void addLayoutComponent(String name, Component comp) {
	}

	public void addLayoutComponent(Component comp, Object constraints) {
		if (constraints instanceof Dimension) {
			Dimension dim = (Dimension)constraints;
			compMap.put(comp, new Dimension(dim.width, dim.height));
		} else if (constraints != null) {
			throw new IllegalArgumentException("cannot add to layout: constraint must be a Dimension");
		}
	}

	public void removeLayoutComponent(Component comp) {
		compMap.remove(comp);
	}

	public Dimension preferredLayoutSize(Container target) {
		return getLayoutSizeImpl(target, false);
	}

	public Dimension minimumLayoutSize(Container target) {
		return getLayoutSizeImpl(target, true);
	}

	public void layoutContainer(Container target) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.insets();
		int hgap = this.gap * fontSize.width / 4;
		int vgap = this.gap * fontSize.height / 8;
		Dimension targetSize = target.size();
		int clientWidth = targetSize.width - insets.left - insets.right - hgap * 2;
		int clientHeight = targetSize.height - insets.top - insets.bottom - vgap * 2;

		Vector compList = new Vector();
		int width = 0;
		int height = 0;

		int count = target.countComponents();
		for (int i = 0; i < count; i++) {
			Component comp = target.getComponent(i);
			if (comp.isVisible()) {
				int compWidth;
				int compHeight;

				Dimension fbuSize = (Dimension)compMap.get(comp);
				if (fbuSize != null) {
					compWidth = fbuSize.width * fontSize.width / 4;
					compHeight = fbuSize.height * fontSize.height / 8;
				} else {
					Dimension compSize = comp.preferredSize();
					compWidth = compSize.width;
					compHeight = compSize.height;
				}

				compList.addElement(comp);
				compList.addElement(new Dimension(compWidth, compHeight));

				if (width < compWidth)
					width = compWidth;
				height += compHeight + vgap;
			}
		}
		height -= vgap;

		int y = insets.top + vgap;
		switch (valign) {
		case TOP:
			break;
		case MIDDLE:
			y += (clientHeight - height) / 2;
			break;
		case BOTTOM:
			y += clientHeight - height;
			break;
		}

		int compCount = compList.size();
		int i = 0;
		while (i < compCount) {
			Component comp = (Component)compList.elementAt(i++);
			Dimension compSize = (Dimension)compList.elementAt(i++);

			int x = insets.left + hgap;
			switch (halign) {
			case LEFT:
				break;
			case CENTER:
				x += (clientWidth - compSize.width) / 2;
				break;
			case RIGHT:
				x += clientWidth - compSize.width;
				break;
			}

			comp.reshape(x, y, compSize.width, compSize.height);
			y += compSize.height + vgap;
		}
	}

	public String toString() {
		String hstr = "";
		switch (halign) {
		case LEFT:
			hstr = "left";
			break;
		case CENTER: 
			hstr = "center";
			break;
		case RIGHT:
			hstr = "right";
			break;
		}
		String vstr = "";
		switch (valign) {
		case TOP:
			vstr = "top";
			break;
		case MIDDLE: 
			vstr = "middle";
			break;
		case BOTTOM:
			vstr = "bottom";
			break;
		}
		return getClass().getName() +
			"[halign=" + hstr + ",valign=" + vstr + ",gap=" + gap + "(fbu)" +
			",baseString='" + baseString + "']";
	}

	// ---- helper methods ----

	protected void construct(int align, int gap, String baseString) {
		switch (align) {
		case LEFT: case CENTER: case RIGHT:
			construct(align, MIDDLE, gap, baseString);
			break;
		case TOP: case MIDDLE: case BOTTOM:
			construct(CENTER, align, gap, baseString);
			break;
		default:
			construct(CENTER, MIDDLE, gap, baseString);
			break;
		}
	}

	protected void construct(int halign, int valign, int gap, String baseString) {
		compMap = new Hashtable();
		this.halign = (LEFT <= halign && halign <= RIGHT) ? halign : CENTER;
		this.valign = (TOP <= valign && valign <= BOTTOM) ? valign : MIDDLE;
		this.gap = gap;
		setBaseStringImpl(baseString);
	}

	protected void setBaseStringImpl(String baseString) {
		this.baseString = (baseString == null || baseString.length() == 0) ? defaultBaseString : baseString;
	}

	protected Dimension getLayoutSizeImpl(Container target, boolean minimum) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.insets();
		int hgap = this.gap * fontSize.width / 4;
		int vgap = this.gap * fontSize.height / 8;

		int width = 0;
		int height = 0;

		int count = target.countComponents();
		for (int i = 0 ; i < count ; i++) {
			Component comp = target.getComponent(i);
			if (comp.isVisible()) {
				int compWidth;
				int compHeight;

				Dimension fbuSize = (Dimension)compMap.get(comp);
				if (fbuSize != null) {
					compWidth = fbuSize.width * fontSize.width / 4;
					compHeight = fbuSize.height * fontSize.height / 8;
				} else {
					Dimension compSize = (minimum) ? comp.minimumSize() : comp.preferredSize();
					compWidth = compSize.width;
					compHeight = compSize.height;
				}

				if (width < compWidth)
					width = compWidth;
				height += compHeight + vgap;
			}
		}
		height -= vgap;

		return new Dimension(
			width + hgap * 2 + insets.left + insets.right,
			height + vgap * 2 + insets.top + insets.bottom);
	}

	protected Dimension getFontSize(Container target) {
		FontMetrics m = target.getFontMetrics(target.getFont());
		int width = m.stringWidth(baseString) / baseString.length();
		if (width <= 0)
			width = 1;
		int height = m.getHeight();
		return new Dimension(width, height);
	}
}
