//
//	FontBaseFlowLayout.java
//
//	version  1.0  98/01/15
//	platform Java 1.1
//	author   Mitsumasa IWATSUKI
//

package layout;

import java.awt.*;
import java.io.Serializable;
import java.util.Hashtable;
import java.util.Vector;

public class FontBaseFlowLayout implements LayoutManager2, Serializable {
	public static final int LEFT = 0;
	public static final int CENTER = 1;
	public static final int RIGHT = 2;
	public static final int TOP = 3;
	public static final int MIDDLE = 4;
	public static final int BOTTOM = 5;

	protected Hashtable compMap;
	protected int halign;
	protected int valign;
	protected int hgap;
	protected int vgap;
	protected String baseString;
	protected static final String defaultBaseString = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

	public FontBaseFlowLayout() {
		construct(CENTER, TOP, 4, 4, defaultBaseString);
	}

	public FontBaseFlowLayout(int align) {
		construct(align, 4, 4, defaultBaseString);
	}

	public FontBaseFlowLayout(int halign, int valign) {
		construct(halign, valign, 4, 4, defaultBaseString);
	}

	public FontBaseFlowLayout(int align, int hgap, int vgap) {
		construct(align, hgap, vgap, defaultBaseString);
	}

	public FontBaseFlowLayout(int halign, int valign, int hgap, int vgap) {
		construct(halign, valign, hgap, vgap, defaultBaseString);
	}

	public FontBaseFlowLayout(String baseString) {
		construct(CENTER, TOP, 4, 4, baseString);
	}

	public FontBaseFlowLayout(int align, String baseString) {
		construct(align, 4, 4, baseString);
	}

	public FontBaseFlowLayout(int halign, int valign, String baseString) {
		construct(halign, valign, 4, 4, baseString);
	}

	public FontBaseFlowLayout(int align, int hgap, int vgap, String baseString) {
		construct(align, hgap, vgap, baseString);
	}

	public FontBaseFlowLayout(int halign, int valign, int hgap, int vgap, String baseString) {
		construct(halign, valign, hgap, vgap, baseString);
	}

	public void setAlignment(int align) {
		switch (align) {
		case LEFT: case CENTER: case RIGHT:
			this.halign = align;
			break;
		case TOP: case MIDDLE: case BOTTOM:
			this.valign = align;
			break;
		}
	}

	public void setAlignment(int halign, int valign) {
		if (LEFT <= halign && halign <= RIGHT)
			this.halign = halign;

		if (TOP <= valign && valign <= BOTTOM)
			this.valign = valign;
	}

	public int getAlignment() {
		return halign;
	}

	public int getHorizontalAlignment() {
		return halign;
	}

	public int getVerticalAlignment() {
		return valign;
	}

	public void setHgap(int hgap) {
		this.hgap = hgap;
	}

	public int getHgap() {
		return hgap;
	}

	public void setVgap(int vgap) {
		this.vgap = vgap;
	}

	public int getVgap() {
		return vgap;
	}

	public void setBaseString(String baseString) {
		setBaseStringImpl(baseString);
	}

	public String getBaseString() {
		return baseString;
	}

	public void addLayoutComponent(String name, Component comp) {
	}

	public void addLayoutComponent(Component comp, Object constraints) {
		if (constraints instanceof Dimension) {
			Dimension dim = (Dimension)constraints;
			compMap.put(comp, new Dimension(dim.width, dim.height));
		} else if (constraints != null) {
			throw new IllegalArgumentException("cannot add to layout: constraint must be a Dimension");
		}
	}

	public void removeLayoutComponent(Component comp) {
		compMap.remove(comp);
	}

	public Dimension preferredLayoutSize(Container target) {
		return getLayoutSizeImpl(target, false);
	}

	public Dimension minimumLayoutSize(Container target) {
		return getLayoutSizeImpl(target, true);
	}

	public Dimension maximumLayoutSize(Container target) {
		return new Dimension(Integer.MAX_VALUE, Integer.MAX_VALUE);
	}

	public float getLayoutAlignmentX(Container target) {
		return 0.5f;
	}

	public float getLayoutAlignmentY(Container target) {
		return 0.5f;
	}

	public void invalidateLayout(Container target) {
	}

	public void layoutContainer(Container target) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.getInsets();
		int hgap = this.hgap * fontSize.width / 4;
		int vgap = this.vgap * fontSize.height / 8;
		Dimension targetSize = target.getSize();
		int clientWidth = targetSize.width - insets.left - insets.right - hgap * 2;
		int clientHeight = targetSize.height - insets.top - insets.bottom - vgap * 2;

		Vector rowList = new Vector();
		int totalHeight = 0;

		int count = target.getComponentCount();
		int i = 0;
		while (i < count) {
			Vector compList = new Vector();
			int rowWidth = 0;
			int rowHeight = 0;

			while (i < count) {
				Component comp = target.getComponent(i);
				if (comp.isVisible()) {
					int compWidth;
					int compHeight;

					Dimension fbuSize = (Dimension)compMap.get(comp);
					if (fbuSize != null) {
						compWidth = fbuSize.width * fontSize.width / 4;
						compHeight = fbuSize.height * fontSize.height / 8;
					} else {
						Dimension compSize = comp.getPreferredSize();
						compWidth = compSize.width;
						compHeight = compSize.height;
					}

					if (rowWidth != 0 && clientWidth < rowWidth + compWidth)
						break;

					compList.addElement(comp);
					compList.addElement(new Dimension(compWidth, compHeight));

					rowWidth += compWidth + hgap;
					if (rowHeight < compHeight)
						rowHeight = compHeight;
				}
				i++;
			}
			rowWidth -= hgap;

			rowList.addElement(compList);
			rowList.addElement(new Dimension(rowWidth, rowHeight));

			totalHeight += rowHeight + vgap;
		}
		totalHeight -= vgap;

		int y = insets.top + vgap;
		switch (valign) {
		case TOP:
			break;
		case MIDDLE:
			y += (clientHeight - totalHeight) / 2;
			break;
		case BOTTOM:
			y += clientHeight - totalHeight;
			break;
		}

		int rowCount = rowList.size();
		i = 0;
		while (i < rowCount) {
			Vector compList = (Vector)rowList.elementAt(i++);
			Dimension rowSize = (Dimension)rowList.elementAt(i++);

			int x = insets.left + hgap;
			switch (halign) {
			case LEFT:
				break;
			case CENTER:
				x += (clientWidth - rowSize.width) / 2;
				break;
			case RIGHT:
				x += clientWidth - rowSize.width;
				break;
			}

			int compCount = compList.size();
			int j = 0;
			while (j < compCount) {
				Component comp = (Component)compList.elementAt(j++);
				Dimension compSize = (Dimension)compList.elementAt(j++);

				comp.setBounds(
					x, y + (rowSize.height - compSize.height) / 2,
					compSize.width, compSize.height);
				x += compSize.width + hgap;
			}
			y += rowSize.height + vgap;
		}
	}

	public String toString() {
		String hstr = "";
		switch (halign) {
		case LEFT:
			hstr = "left";
			break;
		case CENTER: 
			hstr = "center";
			break;
		case RIGHT:
			hstr = "right";
			break;
		}
		String vstr = "";
		switch (valign) {
		case TOP:
			vstr = "top";
			break;
		case MIDDLE: 
			vstr = "middle";
			break;
		case BOTTOM:
			vstr = "bottom";
			break;
		}
		return getClass().getName() +
			"[halign=" + hstr + ",valign=" + vstr +
			",hgap=" + hgap + "(fbu),vgap=" + vgap + "(fbu)" +
			",baseString='" + baseString + "']";
	}

	// ---- helper methods ----

	protected void construct(int align, int hgap, int vgap, String baseString) {
		switch (align) {
		case LEFT: case CENTER: case RIGHT:
			construct(align, TOP, hgap, vgap, baseString);
			break;
		case TOP: case MIDDLE: case BOTTOM:
			construct(CENTER, align, hgap, vgap, baseString);
			break;
		default:
			construct(CENTER, TOP, hgap, vgap, baseString);
			break;
		}
	}

	protected void construct(int halign, int valign, int hgap, int vgap, String baseString) {
		compMap = new Hashtable();
		this.halign = (LEFT <= halign && halign <= RIGHT) ? halign : CENTER;
		this.valign = (TOP <= valign && valign <= BOTTOM) ? valign : TOP;
		this.hgap = hgap;
		this.vgap = vgap;
		setBaseStringImpl(baseString);
	}

	protected void setBaseStringImpl(String baseString) {
		this.baseString = (baseString == null || baseString.length() == 0) ? defaultBaseString : baseString;
	}

	protected Dimension getLayoutSizeImpl(Container target, boolean minimum) {
		Dimension fontSize = getFontSize(target);
		Insets insets = target.getInsets();
		int hgap = this.hgap * fontSize.width / 4;
		int vgap = this.vgap * fontSize.height / 8;

		int width = 0;
		int height = 0;

		int count = target.getComponentCount();
		for (int i = 0 ; i < count ; i++) {
			Component comp = target.getComponent(i);
			if (comp.isVisible()) {
				int compWidth;
				int compHeight;

				Dimension fbuSize = (Dimension)compMap.get(comp);
				if (fbuSize != null) {
					compWidth = fbuSize.width * fontSize.width / 4;
					compHeight = fbuSize.height * fontSize.height / 8;
				} else {
					Dimension compSize = (minimum) ? comp.getMinimumSize() : comp.getPreferredSize();
					compWidth = compSize.width;
					compHeight = compSize.height;
				}

				width += compWidth + hgap;
				if (height < compHeight)
					height = compHeight;
			}
		}
		width -= hgap;

		return new Dimension(
			width + hgap * 2 + insets.left + insets.right,
			height + vgap * 2 + insets.top + insets.bottom);
	}

	protected Dimension getFontSize(Container target) {
		FontMetrics m = target.getFontMetrics(target.getFont());
		int width = m.stringWidth(baseString) / baseString.length();
		if (width <= 0)
			width = 1;
		int height = m.getHeight();
		return new Dimension(width, height);
	}
}
